/**
  *  \file client/scr-ufo.cc
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/screens.h"
#include "game/global.h"
#include "util/nls.h"
#include "ui/layout.h"
#include "gfx/palette.h"
#include "game/formula.h"
#include "cpluslib/format.h"
#include "ui/util.h"
#include "client/tiles/objheader.h"
#include "client/tiles/ufoinfo.h"
#include "int/if/userif.h"
#include "client/widgets/objcontrol.h"
#include "ui/widgets/button.h"
#include "ui/window.h"
#include "ui/uiglobal.h"

namespace {
    class WUfoInfoWindow : public UIWindow {
        ObjectHolder h;
        GObjectType& type;
        void init();
     public:
        WUfoInfoWindow(GObjectType& type);
    };
}

/***************************** WUfoInfoWindow ****************************/

WUfoInfoWindow::WUfoInfoWindow(GObjectType& type)
    : UIWindow(_("Ufo Information"), BLUE_WINDOW, COLOR_GRAY),
      type(type)
{
    init();
}

void
WUfoInfoWindow::init()
{
    setLayout(UIVBoxLayout::instance5);

    UIGroup& g1 = h.add(new UIGroup(UIHBoxLayout::instance5));

    g1.add(h.add(new UIButton(0, _("Close"), SDLK_ESCAPE)).addStop());
    g1.add(h.add(new UIButton(1, _("Go To"), SDLK_RETURN)).addStop());
    g1.add(h.add(new UISpacer()));

    add(h.add(new WObjectTypeHeaderTile(type)));
    add(h.add(new WUfoInfoTile(type)));
    add(g1);
    add(h.add(new WObjectTypeControl(type)));

    pack();
}


void
doUfoScreen()
{
    if (getDisplayedTurn().getCurrentUniverse().getUfos().getNumUfos() == 0)
        return;

    WUfoInfoWindow win(global_ufo_type);
    ui_root->centerWidget(win);
    int result = ui_root->runModal(win);
    if (result == 1) {
        // "Go to"
        IntUserInterfaceBinding::getCurrentInstance()->doGoToChart(global_ufo_type.getUfo().getPos());
    }
}

/** Find nearest Ufo. Returns 0 if none found, otherwise slot for GUfoType. */
int
findNearestUfo(GPoint pt)
{
    // FIXME: this has implicit knowledge of the Id assignment scheme of GUfoType
    int found = 0;
    long dist2 = 0;
    bool inside = false;
    int slot = 0;
    const GUfoContainer& ufos = getDisplayedTurn().getCurrentUniverse().getUfos();
    for (GUfoContainer::const_iterator i = ufos.begin(); i != ufos.end(); ++i) {
        long ndist2 = distanceSquared(i->getPos(), pt);
        bool ninside = (ndist2 <= square(i->getRadius()));
        ++slot;
        /* Pick this Ufo, if
           - it's the first one
           - it's closer than our previous choice
           - we're inside it but not inside the previous one */
        if (found <= 0
            || ndist2 < dist2
            || (ninside && !inside))
        {
            found  = slot;
            dist2  = ndist2;
            inside = ninside;
        }
    }
    return found;
}
