/**
  *  \file game/actions/cargocost.h
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_ACTIONS_CARGOCOST_H
#define PCC_V2_GAME_ACTIONS_CARGOCOST_H

#include "cpluslib/types.h"
#include "cpluslib/sigslot.h"
#include "game/cost.h"
#include "game/cargotype.h"
#include "game/transact.h"
#include "game/error.h"

class GCargoContainer;

//! Transaction which costs some cargo.
class GCargoCostTransaction : public GTransaction {
    GCargoContainer& container;

    GCost cost;
    GCost reserved;

    void update();

 protected:
    void changeCost(GCargoType type, int32_t amount);
    void changeCost(const GCost& cost);
    void setCost(GCargoType type, int32_t amount);
    void setCost(const GCost& cost);

    void commit();
    void cancel();

 public:
    GCargoCostTransaction(GCargoContainer& container);
    ~GCargoCostTransaction();

    bool isValidCost() const;

    GCost   getCost() const;
    int32_t getCost(GCargoType type) const;
    int32_t getRemaining(GCargoType type) const;
    int32_t getMissing(GCargoType type) const;
    int32_t getReserved(GCargoType type) const;

    GError getError();

    GCargoContainer& getContainer() const;

    Signal0 sig_cost_changed;
};

/** Access to container used by this transaction. */
inline GCargoContainer&
GCargoCostTransaction::getContainer() const
{
    return container;
}

#endif
