/**
  *  TEST: directory and command-line handling
  *
  *  This tests some basic operations; in particular, this also
  *  exercises command line and character set handling.
  */

#include <iostream>
#include "io/dirs.h"
#include "arch/platform.h"
#include "util/except.h"
#include "util/modinit.h"

int main(int /*argc*/, char** argv)
{
    performInitialisation(OnStartup, mod_base_console);
    argv = Platform::convertArgumentsToUtf8(argv);
    while (const char* name = *++argv) {
        std::cout << name << ":\n";

        try {
            Ptr<Directory> dir = Directory::openDir(name);
            Ptr<Enumeration<Ptr<DirectoryEntry> > > e = dir->getEntries();
            while (e->hasMoreElements()) {
                Ptr<DirectoryEntry> entry = e->getNextElement();
                std::cout << "  " << entry->getName() << "\n";
                if (entry->getFileType() == DirectoryEntry::FT_FILE) {
                    std::cout << "    file, " << entry->getSize() << " bytes\n";
                    try {
                        Ptr<Stream> s = entry->openFile(Stream::C_READ);
                    }
                    catch (FileProblemException& e) {
                        std::cout << "      error '" << e.what() << "', file '" << e.getFileName() << "'\n";
                    }
                } else if (entry->getFileType() == DirectoryEntry::FT_DIRECTORY) {
                    std::cout << "    directory\n";
                } else {
                    std::cout << "    unknown\n";
                }
            }
        }
        catch (FileProblemException& e) {
            std::cout << "  error '" << e.what() << "', file '" << e.getFileName() << "'\n";
        }
    }
}
